use crate::{to_home, Authenticated, State};
use actix_session::Session;
use actix_web::{web, HttpResponse};

pub type LogoutPageArgs = (Authenticated, web::Data<State>);

pub type LogoutArgs = (Authenticated, web::Data<State>, Session);

pub struct LogoutState {
    state: State,
}

pub fn logout_page((_, state): LogoutPageArgs) -> LogoutState {
    LogoutState::new(&state)
}

pub fn logout((_, state, session): LogoutArgs) -> HttpResponse {
    crate::extractors::UserData::remove(&session);

    to_home(&state)
}

impl LogoutState {
    fn new(state: &State) -> Self {
        LogoutState {
            state: state.clone(),
        }
    }

    pub fn button<'a>(&self, btn: &'a hyaenidae_toolkit::Button) -> &'a hyaenidae_toolkit::Button {
        btn.form(&self.logout_path())
    }

    pub(crate) fn logout_path(&self) -> String {
        self.state.pages.logout_path()
    }
}
