use crate::{
    apub::actions::{CreateBlock, DeleteBlock},
    recover, Action, Context, Error, RecoverableError, Required,
};
use activitystreams::prelude::*;

pub(crate) fn block(
    block: &activitystreams::activity::Block,
    context: &Context,
) -> Result<Result<Box<dyn Action>, RecoverableError>, Error> {
    let id = block.id_unchecked().req()?;

    // Double create
    if context.apub.object(id)?.is_some() {
        return Err(Error::Invalid);
    }

    let actor_id = block.actor()?.as_single_id().req()?;
    let object_id = block.object().as_single_id().req()?;

    let published = block.published().req()?;

    let actor = recover!(actor_id, context.apub.id_for_apub(actor_id)?);
    let actor = actor.profile().req()?;

    if !context.store.profiles.by_id(actor)?.is_some() {
        return Err(Error::Invalid);
    }

    let object = recover!(object_id, context.apub.id_for_apub(object_id)?);
    let object = object.profile().req()?;

    if !context.store.profiles.by_id(object)?.is_some() {
        return Err(Error::Invalid);
    }

    if let Some(true) = context.store.profiles.is_local(object)? {
        return Ok(Ok(Box::new(CreateBlock {
            apub_id: id.to_owned(),
            blocked_profile: object,
            blocked_by_profile: actor,
            published: published.into(),
        })));
    }

    Err(Error::Invalid)
}

pub(crate) fn undo_block(
    undo: &activitystreams::activity::Undo,
    block: &activitystreams::activity::Block,
    context: &Context,
) -> Result<Result<Box<dyn Action>, RecoverableError>, Error> {
    let undo_actor = undo.actor()?.as_single_id().req()?;
    let block_actor = block.actor()?.as_single_id().req()?;

    if block_actor != undo_actor {
        return Err(Error::Invalid);
    }

    let block_id = block.id_unchecked().req()?;

    let id = recover!(block_id, context.apub.id_for_apub(block_id)?);
    let id = id.block().req()?;

    Ok(Ok(Box::new(DeleteBlock { block_id: id })))
}
