use crate::{
    apub::actions::{CreateBlock, DeleteBlock, DeleteFollow, RejectFollowRequest},
    Action, Context, Error,
};

impl Action for CreateBlock {
    fn perform(&self, context: &Context) -> Result<(), Error> {
        let block = context.store.view.blocks.new(
            self.blocked_profile,
            self.blocked_by_profile,
            self.published,
        )?;

        context.apub.block(&self.apub_id, block.id)?;

        if let Ok(Some(follow_request_id)) = context
            .store
            .view
            .follow_requests
            .by_forward(self.blocked_profile, self.blocked_by_profile)
        {
            Action::perform(&RejectFollowRequest { follow_request_id }, context)?;
        }

        if let Ok(Some(follow_request_id)) = context
            .store
            .view
            .follow_requests
            .by_backward(self.blocked_by_profile, self.blocked_profile)
        {
            Action::perform(&RejectFollowRequest { follow_request_id }, context)?;
        }

        if let Ok(Some(follow_id)) = context
            .store
            .view
            .follows
            .by_forward(self.blocked_profile, self.blocked_by_profile)
        {
            Action::perform(&DeleteFollow { follow_id }, context)?;
        }

        if let Ok(Some(follow_id)) = context
            .store
            .view
            .follows
            .by_backward(self.blocked_by_profile, self.blocked_profile)
        {
            Action::perform(&DeleteFollow { follow_id }, context)?;
        }

        Ok(())
    }
}

impl Action for DeleteBlock {
    fn perform(&self, context: &Context) -> Result<(), Error> {
        context.store.view.blocks.remove(self.block_id)?;
        Ok(())
    }
}
