use crate::{
    apub::actions::{CreateReact, DeleteReact},
    Action, Context, Error, Required,
};

impl Action for CreateReact {
    fn perform(&self, context: &Context) -> Result<(), Error> {
        let react = context.store.reacts.create(
            self.submission_id,
            self.profile_id,
            self.comment_id,
            &self.react,
            self.published,
        )?;

        context.apub.react(&self.apub_id, react.id())?;

        let notifier_id = if let Some(comment_id) = self.comment_id {
            context
                .store
                .comments
                .by_id(comment_id)?
                .req()?
                .profile_id()
        } else {
            context
                .store
                .submissions
                .by_id(self.submission_id)?
                .req()?
                .profile_id()
        };

        if let Ok(Some(true)) = context.store.profiles.is_local(notifier_id) {
            context
                .store
                .view
                .reacts
                .new(notifier_id, react.id(), self.published);
        }

        Ok(())
    }
}

impl Action for DeleteReact {
    fn perform(&self, context: &Context) -> Result<(), Error> {
        context.store.reacts.delete(self.react_id)?;
        context.store.view.reacts.remove(self.react_id);

        Ok(())
    }
}
