#[derive(Clone, Copy, Debug)]
pub enum InputKind {
    Text,
    TextArea,
    Number,
    Password,
}

#[derive(Debug, Default)]
pub struct Input {
    pub(crate) name: String,
    pub(crate) kind: InputKind,
    pub(crate) value: Option<String>,
    pub(crate) placeholder: Option<String>,
}

impl Input {
    pub fn new(name: &str, kind: InputKind) -> Self {
        Input {
            name: name.to_owned(),
            kind,
            ..Default::default()
        }
    }

    pub fn kind(&mut self, kind: InputKind) -> &mut Self {
        self.kind = kind;
        self
    }

    pub fn value(&mut self, value: &str) -> &mut Self {
        self.value = Some(value.to_owned());
        self
    }

    pub fn placeholder(&mut self, placeholder: &str) -> &mut Self {
        self.placeholder = Some(placeholder.to_owned());
        self
    }

    pub fn value_opt(&mut self, value: Option<String>) -> &mut Self {
        self.value = value;
        self
    }

    pub(crate) fn is_textarea(&self) -> bool {
        match self.kind {
            InputKind::TextArea => true,
            _ => false,
        }
    }

    pub(crate) fn kind_str(&self) -> &'static str {
        use InputKind::*;
        match self.kind {
            Text => "text",
            TextArea => "textarea",
            Number => "number",
            Password => "password",
        }
    }
}

impl Default for InputKind {
    fn default() -> Self {
        InputKind::Text
    }
}
